//Changes frequency setting in setup file; triggers wavegen.c to use that information to 
//create a waveform of appropriate size; uses eloader.c to send ethernet packets to waveform 
//mirror board--populating the board's memory with generated data and initiating the 
//pulse-echo process; and, finally, surveying the oscilloscope over a UART connection to 
//determine the amplitude of the resulting waveform produced by the waveform mirror board. 
//This process is repeated over and over again to determine the effect of frequency on the 
//amplitude of test wave patterns.
//Boston University Electronics Design Facility--Conor DuBois

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "sio_cmd.h"

double verbose_send(char * command, int feed);

main(int argc, char * argv[])
{
	char buffer[200];
        int feed, repetition;
	FILE * setup;
	FILE * data;
	
	data = fopen("frequency_info.dat", "w+");

	if((feed = sio_open("/dev/ttyS0")) < 0) 
	{
		printf("Error opening serial port\n");
		exit(1);
	}
	
	buffer[0] = '\0';
	strcat(buffer, "CORS EO, \"\\r\\n$\"\n");
	sio_shout(feed, buffer);

	for(repetition = 1; repetition < 72; repetition += 1)
	{
		setup = fopen("setup.ini", "a"); 
		fprintf(setup, "--freq %d\n", repetition * repetition * 5000);
		fclose(setup);
		fprintf(data, "%d\t", repetition * repetition * 5);
		buffer[0] = '\0';
		strcat(buffer, "TRMD AUTO\n");
		sio_shout(feed, buffer);
		buffer[0] = '\0';
		strcat(buffer, "TRMD SINGLE\n");
		sio_shout(feed, buffer);
		system("./wavegen.exe -I setup.ini | ./eloader.exe");
		buffer[0] = '\0';
		strcat(buffer, "C3:PAVA? AMPL\n");
		fprintf(data, "%f\n", verbose_send(buffer, feed));
	}

	fclose(data);
	system("gnuplot instructions2.txt");
}

double verbose_send(char * command, int feed)
{
	char * response;
	double voltage = 0.0;
	printf("Sending Command: %s", command);
	response = sio_cmd(feed, command);
	command[0] = '\0';
	if(response == NULL)
		strcat(command, "sio_cmd() error\n");
	else
		sscanf(response, "%24c%lf", command, &voltage);
//	return voltage;
	return 20.0 * log10(voltage);
}
