//Waveform generator--produces data sets representing various types waves based on options
//chosen from the command line or terminal.
//Boston University Electronics Design Facility--Conor DuBois

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

int parse(int previous, char * word);
int sinewave(int amplitude, int pulse, int index, int period);
int pulsewave(int amplitude, int pulse, int index, int period);
int toothwave(int amplitude, int pulse, int index, int period);
int squarewave(int amplitude, int pulse, int index, int period);
int trianglewave(int amplitude, int pulse, int index, int period);
int standard(int data, int bits);
int irregular(int data, int bits);
int biased(int data, int bits);

int main(int argc, char**argv)
{
   int index, next, data;
   double junk[6] = {0.0, 10000000.0, 1000.0, 1.0, 1.0, 0.0};
   int bits = 12;
   int offset = 0;
   int period = 100;
   int pulse = 20;
   int amplitude = 2047;
   int phase = 0;
   int duration = 1000;
   int active = 0;
   int lag = 0; 
   int last = -1;
   int filing = 0;
   char recent[32];
   FILE * setup;
   char * format = "%d\n";
   FILE * output = stdout;
   int (*generator)(int, int, int, int) = &sinewave;
   int (*mutator)(int, int) = &standard;

   for(index = 1; index < argc;)
   {
      if(filing)
         if(fscanf(setup, "%s", recent) == EOF)
         {
            fclose(setup);
            filing = 0;
            last = -1;
         }
      if(!filing)
         strncpy(recent, argv[index], 32);

      next = parse(last, recent);
      if(next != -1 && last == next)
      {
         if(next == 0)
            output = fopen(recent, "w+");
         else if(next == 1)
            duration = atoi(recent);
         else if(next == 2)
            offset = atoi(recent);
         else if(next == 3)
            junk[2] = 1.0 / atof(recent);
         else if(next == 4)
            junk[4] = atof(recent);
         else if(next == 5)
            amplitude = atoi(recent);
         else if(next == 6)
         {
            bits = atoi(recent);
            amplitude = (1 << (bits - 1)) - 1;
         }
         else if(next == 7)
            junk[1] = atof(recent);
         else if(next == 8)
            junk[2] = atof(recent);
         else if(next == 9)
            junk[5] = atof(recent);
         else if(next == 10)
            junk[3] = atof(recent);
         else if(next == 11)
            phase = atoi(recent);
         else if(next == 12)
         {
            if(!strcmp(recent, "tri"))
               generator = &trianglewave;
            else if(!strcmp(recent, "sqr"))
               generator = &squarewave;
            else if(!strcmp(recent, "sin"))
               generator = &sinewave;
            else if(!strcmp(recent, "saw"))
               generator = &toothwave;
            else if(!strcmp(recent, "pls"))
               generator = &pulsewave;
         }            
         else if(next == 13)
         {
            if(!strcmp(recent, "bin"))
               format = "%c\n";
            else if(!strcmp(recent, "hex"))
               format = "%X\n";
            else if(!strcmp(recent, "dec"))
               format = "%d\n";
         }
	 else if(next == 14)
         {
            if(!strcmp(recent, "uns"))
            {
               mutator = &biased;
               offset = 1 << (bits - 1);
            }
            else if(!strcmp(recent, "twos"))
               mutator = &standard;
            else if(!strcmp(recent, "ones"))
               mutator = &irregular;
         }
         else
         {
            if(setup != NULL)
               fclose(setup);
            setup = fopen(argv[index], "r");
            filing = 1;
         }
      }
      else
         last = next;
      if(!filing)
         index += 1;
   }
   period = (int)(.5 + (junk[1]/junk[2]));
   active = (int)(.5 + (junk[1]*junk[3]));
   lag = (int)(.5 + (junk[1]*junk[5]));
   pulse = (int)(.5 + (junk[1]*junk[4]));

   for(index = 0; index < duration; index += 1)
   {
      if(index < lag)
         data = 0;
      else if(index < lag + active)
         data = generator(amplitude, pulse, (index + phase*period/360 - lag), period);
      else
         data = 0;
      data += offset;
      data = mutator(data, bits);
      fprintf(output, format, data);
   }
   fclose(output);
}

int sinewave(int amplitude, int pulse, int index, int period)
{
   return amplitude * sin(2.0 * M_PI * ((double)index)/((double)period));
}

int pulsewave(int amplitude, int pulse, int index, int period)
{
   return (index % period) < pulse? amplitude : 0; 
}

int toothwave(int amplitude, int pulse, int index, int period)
{
   double coefficient = ((double)index)/((double)period)-(double)(index/period);
   return -amplitude + (amplitude << 1) * coefficient;
}

int squarewave(int amplitude, int pulse, int index, int period)
{
   return ((index << 1)/period) & 1? -amplitude : amplitude;
}

int trianglewave(int amplitude, int pulse, int index, int period)
{
   double coefficient = ((double)index)/((double)(period>>1)) - (double)(index/(period>>1));
   return (index / (period >> 1)) & 1? -amplitude + (amplitude << 1) * coefficient : amplitude - (amplitude << 1)*coefficient;
}

int irregular(int data, int bits)//send one's complements number of given bit length
{
   return data < 0? data < 1-(1<<(bits-1))? 1<<bits : ~(-data & ((1<<(bits-1))-1)) : data > (1<<(bits-1))-1? (1<<(bits-1))-1 : data & ((1<<(bits-1))-1);
}

int standard(int data, int bits)//send two's complements number of given bit length
{
   return data < 0? data < -(1<<(bits-1))? 1<<bits : data & ((1<<bits)-1) : data > (1<<(bits-1))-1? (1<<(bits-1))-1 : data & ((1<<(bits-1))-1);
}

int biased(int data, int bits)//send unsigned numbers
{
   return data < 0? 0 : data > (1<<bits)-1? (1<<bits)-1 : data & ((1<<bits)-1);
}

int parse(int previous, char * word)
{
   if(!strcmp(word, "-O") || !strcmp(word, "--outfile"))
      return 0;
   if(!strcmp(word, "-r") || !strcmp(word, "--reclength"))
      return 1;
   if(!strcmp(word, "-d") || !strcmp(word, "--dcoff"))
      return 2;
   if(!strcmp(word, "-P") || !strcmp(word, "--period"))
      return 3;
   if(!strcmp(word, "-W") || !strcmp(word, "--pulswidth"))
      return 4;
   if(!strcmp(word, "-a") || !strcmp(word, "--amp"))
      return 5;
   if(!strcmp(word, "-b") || !strcmp(word, "--bitdepth"))
      return 6;
   if(!strcmp(word, "-s") || !strcmp(word, "--samprate"))
      return 7;
   if(!strcmp(word, "-f") || !strcmp(word, "--freq"))
      return 8;
   if(!strcmp(word, "-D") || !strcmp(word, "--delay"))
      return 9;
   if(!strcmp(word, "-t") || !strcmp(word, "--time"))
      return 10;
   if(!strcmp(word, "-p") || !strcmp(word, "--phase"))
      return 11;
   if(!strcmp(word, "-w") || !strcmp(word, "--wavtyp"))
      return 12;
   if(!strcmp(word, "-o") || !strcmp(word, "--outform"))
      return 13;
   if(!strcmp(word, "-c") || !strcmp(word, "--coding"))
      return 14;
   if(!strcmp(word, "-I") || !strcmp(word, "--infile"))
      return 15;
   if(!strcmp(word, "--version"))
   {
      printf("Version 1\n");
      exit(0);
   }
   if(!strcmp(word, "--help"))
   {
      printf("Input option switches (like -w) followed by parameters to\nalter the waveform produced by this program.\nSee http://docs.google.com/View?id=dc392ct7_17cngpgwcw for full explanation.\n");
      exit(0);
   }
   else
      return previous;
}
